/*
	data structures for pdl (packet descriptor langage

*/

#include "xsetjmp.h"	/* setjmp.h + protypes */

#ifndef eextern
#define eextern extern
#endif

void pr_error(char *);

/*
	nd_type field values
*/
#define IS_FLD 		0	/*this is a field of data bits */
#define IS_NAME 	1	/*this is an items name*/
#define IS_PROTO 	2	/*this is a protocol*/
#define IS_INDGSUB  3	/* goto intirect thru a register */
#define IS_LOOPC 	4	/*repeat this count times*/
#define IS_LOOPE 	5	/*repeat till out of data*/
#define IS_EOF 		6	/*the packet should end here*/
#define IS_SPECIAL 	7 	/*call a special routine*/
#define IS_END 		8	/* end of protocol */
#define IS_GOSUB 	9	/* gosub to pdl routines */
#define IS_ZERO	   10	/* zero a register */
#define IS_BEGIN   11	/* name of this record */
#define IS_NUMBER_OF 12	/* number of opcodes*/

#define CMD(xxx) (xxx<<12)	/* map fake command bits to real ones */

/*
	nd_print values
*/
#define HEX 	0x00000000	/* print field in hex */
#define DEC 	0x10000000	/* print field in decimal */
#define NOPRINT 0x20000000	/* don't print at all */
#define CNT_STR 0x30000000	/* print a counted ascii string */
#define IP_ADR  0x40000000  /* an ip address */

/* map the above fake names into real values */
#define PR_REAL(xxx) ((xxx)>>28)

#define VIND(xxx) (((uint32)xxx)<<12)	/* load value inderectly */
#define VLOAD(xxx) ((xxx)<<7)	/* store value into a register */

#define SIND(xxx) (0x40|(xxx))	/* load a size inderectly */

#define CNTR(xxx) (((uint32)xxx)<<17)	/*setup a pdl counter*/

typedef struct {
	uint32 nd_print:4,	/* how to print */
	 	   nd_resv1:6,	/* unused, reserved for expansion */
	 	   nd_cntr:5,	/* what counter this is or zero */
		   nd_indreg:5,	/* indirect (value) register */
		   nd_lodreg:5,	/* load field value into this register */
		   nd_indsize:1, /* size is indirect (nd_size is register) */
	 	   nd_size:6;	/* item size */
}nd_fvalue;	/* define nd_value of a 'field' */

#define CNT_shift (5)
#define CNTR_noprint_real (1)
#define CNTR_noprint	(CNTR_noprint_real<<CNT_shift)
#define CNTR_reject_real (2)
#define CNTR_reject		(CNTR_reject_real<<CNT_shift)
#define CNTR_scope_real	(4)
#define CNTR_scope		(CNTR_scope_real<<CNT_shift)
#define CNTR_noreject	(0)
#define CNTR_print		(0)
#define CNTR_noscope	(0)

/*
	a nd record in a resource
*/
struct nd_res_R {
	uint16 nd_type:4,		/*what kind of node this is*/
		   nd_resrv:4,		/*unused, reserved for expansion*/
		   nd_cntr_flags:3,
		   nd_genreg:5;		/*register for most opcodes*/
	uint32 nd_value;		/*value for names and protocols*/
	uint16 nd_str;			/*string pointer or zero*/
	uint16 nd_down;			/*down pointer or zero*/
};
typedef struct nd_res_R nd_res;

/*
	the real nd_rec format, a fake one is used in laying out the
	data to make the macros for data layout simpler
*/
struct nd_rec_R{
	uint16 nd_type:4,		/* what kind of node this is */
		   nd_resrv:4,		/*unused, reserved for expansion*/
		   nd_cntr_flags:3,
		   nd_genreg:5;		/* register for most opcodes */
	uint32 nd_value;	/* value for names and protocols */
	struct nd_rec_R *nd_down; /* nd_record for gosub,protocol etc. */
	char *nd_name;		/* the fields name */

	uint16 meoff;		/* opcode offset for me */
	uint16 choff;		/* offset into strings of nd_name */
	uint16 dnoff;		/* opcode offset for down */
};

#define PDL_REG_MAX 31 /* the maximum register number */
typedef struct nd_rec_R nd_rec,*nd_rec_pt;


/*
	the fake node structure
*/
struct pdl_node_R {
	uint16 pn_flag1;	/* per node flags */
	uint32 pn_value;	/* node value */
	struct pdl_node_R *pn_down; /* pointer to what makes up this node */
	char *pn_name;		/* the text name of this node */

	uint16 meoff;		/* opcode offset for me */
	uint16 choff;		/* offset into strings of nd_name */
	uint16 dnoff;		/* opcode offset for down */
};

typedef struct pdl_node_R pdl_node,*pdl_node_pt;

/*
	define a bit field
*/
#define field(fld_name,fld_size,fld_radix) \
	{CMD(IS_FLD),((fld_radix)|(fld_size)),NIL,fld_name},

/*
	name a bit pattern
*/
#define name(nam_name,nam_value) \
	{CMD(IS_NAME),nam_value,NIL,nam_name},

#define cnt_name(nam_name,nam_value,cnt_flags) \
	{CMD(IS_NAME)|(cnt_flags),nam_value,NIL,nam_name},

/*
	define a sub protocol
*/
#define protocol(prot_name,prot_adr,prot_value,reg) \
	{CMD(IS_PROTO)|(reg),(prot_value),(pdl_node_pt)&prot_adr,prot_name},

#define loop_till_end(lc_routine) \
	{CMD(IS_LOOPE),0,(pdl_node_pt)&lc_routine,NIL},

#define loop_count(loop_value,lc_routine) \
	{CMD(IS_LOOPC)|loop_value,0,(pdl_node_pt)&lc_routine,NIL},

#define pdl_begin(xxxx,xxxx_name) pdl_node xxxx[]={ \
	{CMD(IS_BEGIN),0,NIL,xxxx_name},

#define pdl_begin_forward(xxxx,xxxx_name) pdl_node xxxx[]={ \
	{CMD(IS_BEGIN),0,NIL,xxxx_name},

#define pdl_extern(xxxx) extern pdl_node xxxx[]; 

#define pdl_end {CMD(IS_END),0,NIL,NIL} };

#define thats_all {CMD(IS_EOF),0,NIL,NIL},

#define pdl_indirect_gosub(reg) {CMD(IS_INDGSUB)|reg,0,NIL,NIL},

#define pdl_gosub(pdl_go) \
	{CMD(IS_GOSUB),0,(pdl_node_pt)&pdl_go,NIL},

#define pdl_special(pdl_go) \
	{CMD(IS_SPECIAL),(pdl_go),NIL,NIL},

#define pdl_zero(reg) {CMD(IS_ZERO)|reg,0,NIL,NIL},

void pdl_print(uint8 *,int,uint16);

/*
	interpretor globals for special functions to use
*/

#define BITS_PER_DATA 8		/* how big a data item is */
#define DATA_IS uint8		/* change to uint16 if all packets word aligned */

typedef struct {
 uint32 pr_last_value;		/* last field value */
 uint32 pr_size;			/* bits left to use in data packet */
 uint8 pr_curb;				/* current bit position for reading data */
 DATA_IS *pr_data;			/* pointer to data */
 uint32 pr_regs[PDL_REG_MAX+1]; /* the pdl registers */
 jmp_buf pr_jmp_buf;		/* error throw for pdl errors */
 nd_res *pr_ip;				/* pr current instruction pointer */
 nd_res *pr_next;			/* pr next instruction pointer */
} pdl_pr_dat,*pdl_pr_dat_pt;

eextern pdl_pr_dat ppg;	/*pdl print globals*/
void do_special(pdl_pr_dat_pt,long);

/* pdl resource header */
struct pdl_reshdr_R {
	uint16 pdl_head;	/*offset for interpreting the header*/
	uint16 pdl_root;	/*code offset for interpreting pdl root*/
};
typedef struct pdl_reshdr_R pdl_reshdr;

/* well known offsets say where pdl code for that type starts
   This will definitly change to something more expandable.  Perhaps
   with each typed geting its own resources.
*/
#define MAX_STARTS 16
#define GETPAK 		0
#define ATALK_root 	1
#define IP_root 	2

/* number of packet counters/selection points, also will change
   to something expandable
*/
#define MAX_COUNTS	16
/* header record at the start of the pdl code segment*/
struct pdl_world_hdr_R {
	uint16 starts[MAX_STARTS];	/*pdl offsets for main protocols*/
	uint16 counts[MAX_COUNTS];	/*pdl offsets for counter fields*/
};
typedef struct pdl_world_hdr_R pdl_world_hdr;