/*
	sc.h - general globals
*/

#ifndef eextern
#define eextern extern
#endif

#include "XintYY.h"
#include "xsetjmp.h"
#include <windowmgr.h>

/* general math macros */
#define max(a,b) (((a)>(b)) ? (a) : (b))
#define min(a,b) (((a)<(b)) ? (a) : (b))

/**********************************************************************
	bomb - fatal error handler
*/
#define STRN_bomb_txt 0		/*STR# resource id for general bomb text*/
#define STRN_BOMB 1			/*STR# resource id for BOMB_... index below*/
#define BOMB_nowindow 	 1	/*can't find a window from the resource file*/
#define BOMB_noctl 		 2	/*can't read a control from the resource file*/
#define BOMB_nores 		 3	/*can't open sp_resource.rsrc*/
#define BOMB_sr_cnf 	 4	/*seer configure failed*/
#define BOMB_sr_gnrl 	 5	/*a seer call failed with an os error*/
#define BOMB_sr_on 		 6	/*seer on call failed*/
#define BOMB_sr_load 	 7	/*seer load call failed*/
#define BOMB_nodrvrfile  8	/*can't open seer_driver file*/
#define BOMB_nodrvropen  9	/*can't open .seer device*/
#define BOMB_sr_getpak  10	/*seer get packet call failed unexpectedly*/
#define BOMB_nomenu 	11	/*can't find a menu in the resource file*/
#define BOMB_string		12  /*can't read a string from the resource file*/
#define BOMB_badwind	13	/*refcon's point to windef's, this ones broken*/
#define BOMB_nopdl		14	/*can't get a pdl resource*/

#define BMB_no_os		0	/*place holder for bomb os err argument*/
#define BMB_no_special	0L	/*place holder for bomb special argument*/
void bomb(int bomb_type,int os_err,long bomb_special);


/**********************************************************************
	alert - non fatal error handler
*/
#define AL_no_windef 0
void do_alert(int);

/************************************************************************
	Magic stuff to allow us to have globals in a desk accessory.
	Lightspeed C uses A4 for accessing globals in desk accessories.
	In some places (control trackers, dialog filters, etc.)
	iSETUP_GLOBAL_REG/iRESTORE_GLOBAL_REG are needed to get the
	right value into a4.  So that we don't need to change any
	source files to make a desk accesory or a program, some of
	this setup happens for programs even though it is not needed.

 real_sr_ar - not really a function, this routine is patched to be the address of
load_global_rtn - load a4 from real_sr_ar, (for a DA) do nothing for a program
*/
void real_sr_a4(void);
void load_global_rtn(void);
#define GLOBAL_REG a4
/*setup / restore for interupt routines*/
#define iSETUP_GLOBAL_REG \
		uint32 savea4; \
		asm { move.l GLOBAL_REG,savea4} \
		load_global_rtn();
#define iRESTORE_GLOBAL_REG asm { move.l savea4,GLOBAL_REG }

/**************************************************************************
	per window variables
*/
/*information for any of our windows*/
#define SCROLL_BAR_PIXLS 16	/*the width of a scroll bar*/

/*besides being in wd_type these WD_... also index WIND resources
   starting at res_base
*/
#define WD_pak 		0	/*this window is a packet window*/
#define WD_scope 	1	/*this window is a scope window*/
#define WD_status 	2	/*this is a seer status window*/
#define WD_free		(-1) /*this windef is not in use*/
#define PRIVATE_SIZE 200 /* *must* be as large as largest private data */
struct windef_R{
 WindowRecord windat;	/*non relocatable data for the window*/
 int wd_type;			/*a WD_... value what kind of window this is*/
 int char_height;
 int char_kern;
 int char_descent;
 int char_width;
 int char_left_inset;		/*where in window to start drawing a character*/
 void (*w_draw)();			/* ()() redraw yourself*/
 void (*w_mouse_down)();	/* ()(EventRecord *) */
 void (*w_activate)();		/* ()() */
 void (*w_deactavate)();	/* ()() */
 void (*w_grow)();			/* ()(Point where) */
 char windef_private[PRIVATE_SIZE]; /* per window type private data*/
};
typedef struct windef_R windef;

/************************************************************************
		function prototypes for global functions
*/
void DrawNum(long);
void doabout(void);
int strlen(char *);
void draw_paks_in_Q(void);
void init_mac(void);
void init_windef(void);
void init_globals(void);
void pdl_opn_res(void);
void packet_filter(void);
void scope_tick(void);

void e_activate(WindowPtr);
void e_deactivate(WindowPtr);
void new_pak_display(void);

void inval_pak_cache(void);

/*status window routines*/
void st_new_windef(void);

/*general packet window routines*/
void pk_new_windef(void);
void sk_new_windef(void);
void check_paks_in_Q(void);
void scroll_draw_packets(long);
void drawp_init(void);
void event_loop(void);

void delete_my_windefs(void);

/*general windef routines*/
int new_windef(int);
void zero(char *,long);

/*grow event and where it ventors to*/
void e_grow(Point);

/* seer menu items */
#define SMENU_flushq 1
void do_seer_menu(int);
void show_grow(void);

/*************************************************************************
	global variables - put them in a record, so we can zero them all
	esaly, and make shure that they are in the same order as listed
	for debugging.
*/
struct globs_R {
	int res_base;			/*first resource id of our resources*/
	jmp_buf seer_exit_lngjmp;	/*throw fatal errors thru here*/
	int bomb_active;		/*to check for recursive errors*/
	windef *wdf_status; 	/*the status windef or NIL if none*/
	windef *wdf_scope;		/*the scope windef or NIL if none*/
	windef *wdf_packet;		/*the packet windef if any*/
	windef *cur_windef;		/*windef for the current port or NIL if none*/
	int num_windefs;		/*how many windefs/windows there are*/
	int win_ref_num;		/*what to put into windowKind for us*/
	windef *all_windefs;
	uint8 *pdl_str_ptr;		/*pdl code data structure handle */
    uint8 *pdl_cod_ptr;
	int seer_refnum;		/*driver reference number for seer*/
	int seer_res_file;		/*resource file number for seer driver file*/
	Ptr sp_header_q;		/*seer header queue*/
 	Ptr sp_data_q;			/*seer data queue*/
	int pdl_res;			/*file number for pdl file*/
};
eextern struct globs_R gl;


/*************************************************************************

	The macros and casts below allow everyone to include this file
	without having to know about the per window private data.  This is
	important since some of the private data requires other include
	files (e.g. ControlMgr.h).  This considerable speeds compiling.
*/

/* given an index into the pdl strings, return a pointer to the string
   note:both a counted (pascal) and null terminated string are available,
   the c string is returned by RES_str. The pascal string by RES_pstr
*/
#define RES_str(sindex) ((char *)(gl.pdl_str_ptr+(sindex)+1))
#define RES_pstr(sindex) ((char *)(gl.pdl_str_ptr+(sindex)))

/* given an index into the pdl code, return a pointer to the code
   block there
*/
#define RES_cod(cindex) ((nd_res *)(((char *)gl.pdl_cod_ptr)+(cindex)))

#define PDL_WORLD ((pdl_world_hdr*)gl.pdl_cod_ptr)

/* SK_wdf - a pointer to the private data area of the scope window
*/
#define SK_wdf ((scope_windef*)&gl.wdf_scope->windef_private)
/* SK_wdf - a pointer to the private data area of the packet
*/
#define PAK_wdf ((pak_windef*)&gl.wdf_packet->windef_private)

#define STW_wdf ((status_windef*)&gl.wdf_status->windef_private)

/*
	CUR_WINDOW - the WindowPtr for the current windef
*/
#define CUR_WINDOW ((WindowPtr)&gl.cur_windef->windat.port)

