/*	sr_rdpak.h - .h file for using the 'seer' code resource to read
	AppleTalk packets permisciously

Change log:

  date     who                   what
-------- -------- ---------------------------------------
03/17/87  wohl    v3.0 have getpak return the hardware crc in the header
                  and adjust the hardware packet length to pretend
                  it isn't after the data (if the crc is present)
03/09/87  wohl    v2.2 add error counters to status block
02/22/87  wohl    create

*/

#include <DeviceMgr.h>

#ifndef uint8
#include "XintYY.h"
#endif

#include "localDeviceMgr.h"		/*get a real cntrlParam definition*/

#define PAK_SIZE 610			/*maximum suported packet size*/

/*seer error status return codes*/
#define SRe_noErr 0	/*success, no error*/
/*the version of seer you said you wanted to SRr_load isn't available*/
#define SRe_badver 1
/*the packet number you asked SRr_getpak for has already been released*/
#define SRe_never 2
/*the packet number you asked SRr_getpak for hasn't arrived yet*/
#define SRe_notyet 3

/*the current values of major_version and minor_version*/
#define SEER_MAJOR 4
#define SEER_MINOR 0

/*The kinds of errors a packet can have*/
#define PE_none 	0	/*the packet didn't overrun, had an abort, good crc*/
#define PE_badcrc	1	/*the packet had an incorrect hardware crc*/
#define PE_havecrc	1   /*<= here means packet has hardware crc*/
#define PE_overrun	2	/*the recieve data interupt routine missed some data*/
#define PE_noabort	3	/*the packet did not end with an abort flag*/
#define PE_max		3	/*largest error kind*/
/*
	seer status structure.
	A pointer to this is returned by the SRr_load call.
	Following these defined fields are seers private data.
	These may be accessed for debugging purposes by examining
	seer_private.h for the format.
*/
#define SR_status_off 0		/*packet recording is off*/
#define SR_status_on 1		/*packet recording is on*/
#define SR_status_dfull 2 	/*recording was on, but data queue filled up*/
#define SR_status_hfull 3 	/*recording was on, but header queue filled up*/
typedef struct {
	int16 status;			/*current seer status*/
	uint32 pre_pak_num;		/*packet number of packet before readpt*/
	int32 h_inuse;			/*number of header slots in use*/
	uint32 h_size;			/*max number of items in header queue*/
	int32 d_inuse;			/*number of data bytes in use*/
	void (*filter_proc)(); 	/*pointer to interupt level filter proc*/
	Ptr filter_data;		/*for filter_proc use*/
} SR_status,*SR_status_pt;

/*the argument block to a pointer to a routine
   Call the routine after loading
   and locking the 'seer' code resource.  If it is ever unlocked
   then call this routine again.
*/
typedef struct {
 	int major_version;				/*[in]*/
 	int minor_version;				/*[out]*/
/*
	*************************************************************
	  If there is a there is a major version mismatch and
	  SRe_badver is returned, fields below here are unchanged.
	*************************************************************
*/
	SR_status_pt status_pt; 	/*[out] pointer to seer status data structure*/
	int header_length;			/*[out] bytes per packet header*/
	int data_overhead;			/*[out] overhead per packet in data queue*/
} SRa_load;

/*data passed  to SRr_init*/
typedef struct {
	char *dbufstart;	/*[in] start of data buffer*/
	uint32 dbuflen;		/*[in] length of data buffer*/
	char *hbufstart;	/*[in] start of header buffer*/
	uint32 hbuflen;		/*[in] length of header buffer*/
	void (*filter_proc)(); 	/*[in]pointer to interupt level filter proc*/
	Ptr filter_data;	/*[in]for filter_proc use*/
} SRa_initq;

/*data passed to/from SRr_getpak*/
typedef struct {
	uint32 pak_num;		/*[in] the number of the desired packet*/
	char *pak_data;		/*[out] a pointer to the packets data or NIL*/
	uint32 pak_time;	/*[out] when the packet came in (not yet implimented)*/
	uint16 pak_err;		/*[out] packet error number*/
	uint16 pak_size;	/*[out] how big the packet is*/
	uint16 pak_crc;		/*[out] the packets crc*/
} SRa_getpak;

#define SRc_goodbye	  (-1)	/*goodbye kiss, the heap is about to be initilized*/
#define SRc_unused		0	/*unused, < SCc_load turns off interupts*/
#define	SRc_killio		1	/*killio does this control entry to stop io*/
#define	SRc_load		2	/*seer has just been loaded, bind absalute addresses*/
#define	SRc_initq		3	/*setup data and header queues*/
#define	SRc_off			4	/*stop collecting data*/
#define	SRc_on			5 	/*start collecting data*/
#define	SRc_getpak		6	/*return a packet*/
#define	SRc_release		7 	/*release the storage used by a packet*/
#define	SRc_clear_queue	8 	/*empty the queue*/

#define SRc_numberof   10	/*the number of SCc_... entries*/

typedef struct {
	uint16 SR_error;
	uint32 SR_user;
	union {
		SRa_load a_load;
		SRa_initq a_initq;
		SRa_getpak a_getpak;
	} x;
}SR_record,*SR_record_pt;

/*the structure of the seer device driver parameter block*/
typedef struct {
	cntrlParam_header	io_CP;	/*the standard device manager header*/
	SR_record_pt	io_SR;	/*a pointer to a seer record*/
	int SR_dummy[9];		/*fill out to 22 bytes of control param*/
} SR_iorecord;
