/*sr_rdpak_private.h - 'seer' internal data structures
*/

/*the current values of the real major_version and minor_version*/
#define SEER_MAJOR_r 4
#define SEER_MINOR_r 0

#include <DeviceMgr.h>

#ifndef max
#define max(a,b) ((a>b) ? (a) : (b))
#define min(a,b) ((a<b) ? (a) : (b))
#endif

typedef void (*rtn_ptr)();		/*a pointer to a routine*/

typedef int (*int_rtn_ptr)();	/*a pointer to a routine returning int*/

/*
	scc hardware error status bits
*/
#define srphfl_GOT_ABORT_bit 7	/*packet ended with a sdlc abort (normal way)*/
#define srphfl_BAD_HCRC_bit  6	/*this packet had a (hardware) crc error*/
#define srphfl_OVERRUN_bit   5	/*this packet had a data overrun*/
#define srphfl_GOT_ABORT 	(1<<srphfl_GOT_ABORT_bit)
#define srphfl_BAD_HCRC 	(1<<srphfl_BAD_HCRC_bit)
#define srphfl_OVERRUN		(1<<srphfl_OVERRUN_bit)
#define ERR_OR_STOP 0xe0	/*any error or stop bit*/

/*

   srphfl_WRAP means this is the last packet in the data queue before
   wraping around,
   so use sd.dq_final to compute the length of this packet, not
   the start of the next packet.  Also wrap the data queue read pointer
   to bufstart opon reading this packet.

*/

#define srphfl_WRAP_bit    8	/*data is last in data queue*/
#define srphfl_ODD_bit     7	/*packet length is odd*/
#define srphfl_ODD			(1<<srphfl_ODD_bit)
#define srphfl_WRAP			(1<<srphfl_WRAP_bit)
#define srphfl_ERR_mask 0x7		/*mask to keep error status bits*/

/*
	this isn't really a routine.  It is space that is patched to be
	the adrress of the 'seer' code segment globals.  Interupt routines
	load a4 from here.  This allows 'seer' to run under switcher (since
	we don't care what A5 and CurentA5 are).
*/
void real_sr_a4(void);

/*
	prototypes for top level seer functions.  These are called by
	a dispatch routine in sr_main
*/
int SRr_load(SR_record *);
int SRr_initq(SR_record *);
int SRr_off(SR_record *);
int SRr_on(SR_record *);
int SRr_getpak(SR_record *);
int SRr_release(SR_record *);
int SRr_clear_queue(SR_record *);


/*format of a packet header*/
typedef struct {
	uint32 flags:8,		/*per packet flags*/
	 	   address:24;	/*the address of the data*/
} pak_hdr;

#define GLOBAL_REG a4
/*setup / restore for interupt routines*/
#define iSETUP_GLOBAL_REG asm { move.l GLOBAL_REG,-(sp) \
		lea real_sr_a4,GLOBAL_REG \
		move.l (GLOBAL_REG),GLOBAL_REG}
#define iRESTORE_GLOBAL_REG asm { move.l (sp)+,GLOBAL_REG }

/*wrap a header queue pointer into the queue*/
#define WRAP_hq_pt(xxxx) { \
 if(xxxx >= sd.hq.bufendpt) \
   xxxx -=  sd.hq.buflength; }

/*
	the format of a queue, header or data
	N.B. The data queue as an aditional pointer sd.dq_final to help
	compute the length of the last data packet before wraping the
	data queue.  The length of data packets is computed by seeing
	where the current packet starts and where the next packet starts.
	Since there isn't a next packet immediatly after the last packet
	before the end of the data queue (there is unused space) we
	use sd.dq_final to point to where the last packet would have gone.

	There is a status bit in each packet (srphfl_WRAP) that says
	to use sd.dq_final to compute this packets size, not the
	start of the next packet.

	Packets can be of odd length but each packet needs to start on
	a even boundry so things like the 16 bit ddp checksum can be loaded
	as integers without geting an address exception. The flag srphfl_ODD
	is set in odd length packets in which case the next packet (or
	sd.dq_final) points one byte later than the real end of the packet
	(allowing one padding byte).
*/
typedef struct {
	char *bufwritept;	/*write to buffer pointer*/
 	char *bufstartpt;	/*pointer to first byte of buffer space*/
 	char *bufendpt;		/*pointer to first byte not in buffer space*/
 	char *bufreadpt;	/*read from buffer pointer*/
 	uint32 buflength;	/*how big the buffer space is (in bytes)*/
} sr_q;

typedef struct {
	SR_status ast;		/*our public status variables*/
	sr_q dq;			/*data queue*/
	/*pointer after last packet before wraping around if srfl_WRAP is set*/
	char *dq_final;
	sr_q hq;			/*header queue*/
	/*save interupt vectors here on startup so we can restore later*/
	/*VBLQElPtr - say char * here so users don't need vertical .h file*/
	char *our_vbl;
	/*old interupt vectors*/
	rtn_ptr old_tranm_int;
	rtn_ptr old_data_vector;
	rtn_ptr old_special_vector;
	rtn_ptr old_external_vectors;
} seer_data;

/*
	allow everone to use this .h file.  In one file do a define of
	eextern to nothing to actualy declare the space.  This allows
	the declaration that allocates the storage to use the same
	text as the external declarations and reduces the chance for
	mismatches.
*/
#ifndef eextern
#define eextern extern
#endif
eextern seer_data sd;			/*seers data*/

#define INTS_OFF 0x2600
#define SCC_interupts_off asm { \
	move sr,-(a7)		/*save current sr*/ \
	move #INTS_OFF,sr 	/*turn off scc interupts*/}

#define SCC_interupts_on asm {move (a7)+,sr}

/*system interupt vectors*/
#define Lvl2_B_transm 0		/*Lvl2DT index for SCC B transmit interupt*/
#define Lvl2_B_data 2		/*Lvl2DT index for SCC B data interupt*/
#define Lvl2_B_special 3	/*Lvl2DT index for SCC B special interupt*/
#define Ext_B 0				/*ExtStsDT index for SCC B external*/

#define bCtl 0				/*offset from SCCRd to b control port*/
#define bData 4				/*offset from SCCRd to b data port*/
#define xCtl 0				/*offset for interpt routine to control*/
#define xData 4

void sr_tranm_int(void);
void sr_special_int(void);
void sr_ext_int(void);
void sr_ints_off(void);
void sr_ints_on(Ptr,rtn_ptr,rtn_ptr,rtn_ptr,rtn_ptr);
void sr_readpacket(void);
