/*
	sp_pdl_print.c - print a data packet by calling the compiled
	pdl code
*/

#include "sc.h"
#include "sc_seer_globals.h"
#include "pdl.h"

#include "OUT.h"			/*how this to actually print strings*/

static char maptoprint[17]="0123456789abcdef";
/*
	and mask for bit sizes
*/
static uint32 bit_mask[33]={
   0,
   0x00000001,0x00000003,0x00000007,0x0000000f,
   0x0000001f,0x0000003f,0x0000007f,0x000000ff,
   0x000001ff,0x000003ff,0x000007ff,0x00000fff,
   0x00001fff,0x00003fff,0x00007fff,0x0000ffff,
   0x0001ffff,0x0003ffff,0x0007ffff,0x000fffff,
   0x001fffff,0x003fffff,0x007fffff,0x00ffffff,
   0x01ffffff,0x03ffffff,0x07ffffff,0x0fffffff,
   0x1fffffff,0x3fffffff,0x7fffffff,0xffffffff
};

/*
	pr_error - process a pdl error
*/
void pr_error(err)
char *err;
{OUT_brk();
 OUT_str(err);		/*print the error message*/
 longjmp(ppg.pr_jmp_buf,1); /*quit pdl*/
}

/*
	load some bits from the data stream
*/
uint32 slurp_bits(int);
uint32 slurp_bits(bitsize)
register int bitsize;
{register uint32 result;
 register int bits_here;
 result=0;					/*start with no bits on in the result*/
 ppg.pr_size -= bitsize;		/*account for bits used from the packet*/
 if(ppg.pr_size<0)
  pr_error("??packet too short");
 while(bitsize>0) {			/*slurp some bits*/
	bits_here=min(bitsize,ppg.pr_curb); /*how many to get from current byte*/
	bitsize -= bits_here;	/*account for the bits we are providing*/
    result <<= bits_here;	/*make room for this chunk*/
  	/*bits from current byte*/
    result |= (bit_mask[bits_here] & ((*ppg.pr_data)>>(ppg.pr_curb-bits_here)));

	/*advance data stream*/
	ppg.pr_curb -= bits_here;	/*get bits out of the current byte*/
	if(ppg.pr_curb==0) {		/*this byte empty?*/
		ppg.pr_data++;			/*advance data pointer*/
		ppg.pr_curb=BITS_PER_DATA; /*reset bits left in this byte*/
 		}
 	}
 return result;
}

/*
	print a number in some base
*/
#define MAX_int_DEC (40)	/*larger than largest integer as a string*/
char *pr_base(char *,uint32,int);
char *pr_base(dec_put,val,base)
register char *dec_put;
register uint32 val;
register int base;
{register uint32 new_val;
 *--dec_put = 0;			/*end the string with a null*/
 do {
	new_val = val / base;	/*pick out a digit*/
 	*--dec_put = maptoprint[val - (new_val * base)];
	val = new_val;			/*remaining bits*/
 } while (val!=0);
 return dec_put;
};
/*
	print a hex number
*/
pr_hex(uint32);
pr_hex(val)
uint32 val;
{char prstr[MAX_int_DEC+1];
 register char *astext;
 astext=pr_base(&prstr[MAX_int_DEC+1],val,16);
 *--astext = '$';			/*hex numbers begin with dollar signs*/
 OUT_str(astext);
}

/*
	print a decicmal number
*/
pr_dec(uint32);
pr_dec(val)
uint32 val;
{char prstr[MAX_int_DEC+1];
 OUT_str(pr_base(&prstr[MAX_int_DEC+1],val,10));
}

/*
	print a string with a count
*/
#define out_ch(xxx) *outpt++ = (xxx)
#define out_ctl(xxx) {out_ch('^');out_ch(xxx);}
pr_cnt_str(void);
pr_cnt_str()
{register int len;
 char buf[540];				/*worst case string size is 2*(2^8)+1*/
 register char *outpt;		/*current pointer into the output buffer*/	
 register char ch;
 len=slurp_bits(8);			/*get the string length*/
 OUT_str("[");
 pr_dec(len);
 OUT_str("]");
 /*make up the string to print*/
 outpt=buf;					/*put characters here*/
 out_ch('"');
 while((--len)>=0) {		/*until the string is printed*/
	ch=slurp_bits(8);		/*get the next character from the input stream*/
    ch &= 0x7f;				/*mask out parity*/
 	if(ch==0x7f)			/*a delete?*/
	  out_ctl('?')			/*yes print this way*/
	else if(ch<' ')			/*less than a space, a control char?*/
	  out_ctl(((ch&0x1f)+'@')) /*yes, print as ^x*/
	else if(ch=='^')		/*an up arrow?*/
	  out_ctl('^')			/*yes, double it*/
	else
	  out_ch(ch);
	}
 out_ch('"');
 out_ch(0);					/*terminate the string*/
 OUT_str(buf);
}

void pr_ip_adr(uint32);
void pr_ip_adr(a_val)
uint32 a_val;
{pr_dec((a_val>>24) & 0xff); /*print a byte*/
 OUT_str(".");
 pr_dec((a_val>>16) & 0xff); /*print a byte*/
 OUT_str(".");
 pr_dec((a_val>>8) & 0xff); /*print a byte*/
 OUT_str(".");
 pr_dec(a_val & 0xff); 	/*print a byte*/
}

/*
	impliment the field opcode
*/
void pdl_field(fld_arg,fld_name)
uint32 fld_arg;
char *fld_name;
{register int prf_size,temp;		/*will contain size in bits of this field*/
 nd_fvalue fld_vals;
 register uint32 pr_value;			/*the value of this field (if a number)*/
 int print_how;
 fld_vals = *((nd_fvalue *)&fld_arg); /*get our bits from nd_size*/

 /*figure the field size*/
 prf_size=fld_vals.nd_size;		/*assume the size is a number*/
 if(fld_vals.nd_indsize)		/*size inderect?*/
   prf_size=ppg.pr_regs[prf_size];	/*yes, load it from a register*/

 if((temp=fld_vals.nd_indreg)!=0) {	/*load value from a register?*/
    pr_value=ppg.pr_regs[temp];			/*yes*/
 	pr_value &=bit_mask[prf_size];	/*keep only bits valid for this size*/
	}
 else
 	pr_value=slurp_bits(prf_size); /*get the bits from the data stream*/


 /*we have the value now, if we want to save it in a register do so*/
 if((temp=fld_vals.nd_lodreg)!=0) /*want to save the value?*/
   ppg.pr_regs[temp]=pr_value; /*yes, do so*/


 ppg.pr_last_value = pr_value;	/*save for 'name' to match*/

 print_how=fld_vals.nd_print;	/*see how to print*/
 if(print_how==PR_REAL(NOPRINT)) /*want to print at all?*/
  return;						/*no, we are done then*/
 
 OUT_brk();						/*break output here*/
 OUT_str(fld_name); 			/*say what the field is*/
 OUT_str("=");

 /*print this field as desired*/
 switch(print_how) {
  case PR_REAL(HEX):
  	pr_hex(pr_value);
  	break;
  case PR_REAL(DEC):
    pr_dec(pr_value);
    break;
  case PR_REAL(NOPRINT):
  	break;
  case PR_REAL(CNT_STR):
	pr_cnt_str();
  	break;
  case PR_REAL(IP_ADR):
  	pr_ip_adr(pr_value);
  	break;
 }
}

void pdl_print(packet,pdl,pdl_siz)
DATA_IS *packet;			/*the raw data to print*/
void (*pdl)();				/*how to print it*/
uint16 pdl_siz;				/*how big the data is (in bits)*/
{ppg.pr_size=pdl_siz;			/*how many bits of data there are*/
 ppg.pr_size *= 8;
 ppg.pr_data=packet;			/*where to read data from*/
 ppg.pr_curb=BITS_PER_DATA;		/*bits left in this data byte*/	

 if(setjmp(ppg.pr_jmp_buf)!=0)	/*set a place to throw errors*/
  return;					/*return from an error*/

 (*pdl)();
}
