/*
	pdl compiler main program
	
	This program reads the data structures in memory that where
	created by sc_pdl_dat.c amd sc_pdl_ip.c  It reformats the memory
	pointer to be segment offsets and writes a resource file of
	the pdl information.
	
	Also a pdl_gen.h is created which is a .h file declaring all
	data structures and defines for the information in the pdl
	data structures.
*/
#define eextern
#include "XintYY.h"
#include "pdl_comp.h"

extern nd_rec pdl_hdr[];		/*pdl root for getpak results*/
extern nd_rec pdl_root[];		/*the root for appletalk packets*/
extern nd_rec ip[];				/*the root for appletalk packets*/
extern nd_rec misc_counts[];	/*counter names*/

void pdl_compl(nd_rec_pt);		/*forward decl since compiler is recursive */

/* note, the string allocator starts at 2 not zero, this is because a
   string offset of 0 is used as NIL
*/
static uint16 str_alloc=2;		/*offset for allocating string data */
static char **str_handle;		/*handle to string data */
static char *str_ptr;			/*pointer while handle is locked*/
static uint16 cod_alloc=0;		/*code segment allocator (segment offset)*/
static char **cod_handle;		/*handle to code data*/
static char *cod_ptr;			/*pointer to code while handle is locked*/
static pdl_world_hdr *pdl_world;	/*code segment header*/
static FILE *hfile;				/* the .h file we are making */
static FILE *lfile;			/* output .lsp file */
static int last_field_printas;

/* prototypes for system functions*/
int vfprintf(FILE *,char *,...);
int fflush(FILE *);
int fclose(FILE *);
int strcmp(char *,char *);
FILE *fopen(char *,char *);

/* algol-w link assertion handler */
#define assert(xxxx,yyyy) {if(!(xxxx)){yyyy;}}

/* xat - assert handler */
void xat(char *,...);
void xat(errmsg,args)
char *errmsg;
int args;
{vfprintf(stdout,errmsg,(char *)&args);
 fprintf(stdout,"\nassertion failed, click mouse to exit\n");
 fflush(stdout);
 SysBeep(10);
 while (!Button());
 ExitToShell();
}

/* Add the string pointed to by cur_nod->nd_name to the string segment.
   Return it's segment offset or zero if there was no string.
   The string segement is a bunch of strings with a count and a terminating
   null.  This is so they may be be used from .c or pascal easily.  The
   null is not included in the count.
*/
uint16 add_str(nd_rec_pt);
uint16 add_str(cur_nod)
register nd_rec_pt cur_nod;
{register int s_len;
 uint16 result;
 char *write_pt;
 if(cur_nod->nd_name==0)		/* have a string pointer?*/
 	return NIL;					/*no, no string then, say so*/
 s_len=strlen(cur_nod->nd_name); /*get the strings length*/
 if(s_len==0)					/*any characters in it?*/
 	return NIL;					/*no, say so*/ 
 result=str_alloc;			/*the string will start at cur string allocator*/
 write_pt = str_ptr;
 write_pt += str_alloc;
 str_alloc+= s_len + 2;		/* use length of string + count byte + null */
 str_alloc +=1;				/*pad up to an even boundry*/	
 str_alloc &= ~1;
 assert(((((uint32)str_alloc)&0x8000)==0),xat("string segment is less than 32k"));

 *write_pt++ = s_len;		/* put out a count */
 BlockMove(cur_nod->nd_name,write_pt,s_len+1); /*copy over the c string */

 return result;
}

/********************************************************************
		define for each opcode type, which fields of the opcode
		are used.
*/
#define HAS_DOWN TRUE
#define HAS_STR	 TRUE
#define HAS_VALU TRUE

#define NOP_DOWN FALSE
#define NOP_STR	 FALSE
#define NOP_VALU FALSE

struct op_rec_R {
	int has_down;
	int has_str;
	int has_valu;
};
typedef struct op_rec_R op_rec;

static op_rec op_recs[IS_NUMBER_OF]={
{NOP_DOWN,HAS_STR,HAS_VALU},	/*IS_FLD*/
{NOP_DOWN,HAS_STR,HAS_VALU},	/*IS_NAME*/
{HAS_DOWN,HAS_STR,HAS_VALU},	/*IS_PROTO*/
{NOP_DOWN,NOP_STR,NOP_VALU},	/*IS_INDGSUB*/
{HAS_DOWN,NOP_STR,NOP_VALU},	/*IS_LOOPC*/
{HAS_DOWN,NOP_STR,NOP_VALU},	/*IS_LOOPE*/
{NOP_DOWN,NOP_STR,NOP_VALU},	/*IS_EOF*/
{NOP_DOWN,NOP_STR,HAS_VALU},	/*IS_SPECIAL*/
{NOP_DOWN,NOP_STR,NOP_VALU},	/*IS_END*/
{HAS_DOWN,NOP_STR,NOP_VALU},	/*IS_GOSUB*/
{NOP_DOWN,NOP_STR,NOP_VALU},	/*IS_ZERO*/
{NOP_DOWN,HAS_STR,NOP_VALU}		/*IS_BEGIN*/
};

/* print any interesting information from a field opcode to the .h file */
void h_IS_FLD(nd_rec_pt,int*);
void h_IS_FLD(cur_nod,bit_offset)
register nd_rec_pt cur_nod;
int *bit_offset;
{nd_fvalue fld_vals;
fld_vals = *((nd_fvalue *)&cur_nod->nd_value);
/*see if we should stop putting fields into the .h file*/
if((*bit_offset<0)||			/*in variable part of a record?*/
   (fld_vals.nd_indsize!=0)|| 	/*this field variable width?*/
   (fld_vals.nd_indreg!=0)||	/*indirect width?*/
   (fld_vals.nd_size==0)) {		/*zero width(cnt_str etc)?*/
 	   	*bit_offset = -1;		/*yes,funny width, allocate no more bits*/
    	return;}
if(strcmp("0",cur_nod->nd_name)==0)	/* an uninteresting name? */
  fprintf(hfile,			/*no, name use unamed bitfield*/
  "unsigned :%d;\n",(int)fld_vals.nd_size);
else
  {char *siz_nam;
   siz_nam="int";		/* assume it is an int */
   if(fld_vals.nd_size>16)
   siz_nam="long";
   if(fld_vals.nd_size==32)
    fprintf(hfile,"unsigned long %s;\n",cur_nod->nd_name);
   else
  	fprintf(hfile,
    	"unsigned %s %s:%d;\n",siz_nam,cur_nod->nd_name,(int)fld_vals.nd_size);
  }/* end char *siz_nam*/
}

static char *print_name[16]={
	"hex",
	"dec",
	"noprint",
	"cnt_str",
	"ip_addr",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?"
};

void lsp_print_field(nd_rec_pt);
void lsp_print_field(cur_nod)
nd_rec_pt cur_nod;
{nd_fvalue fld_vals;
 fld_vals = *((nd_fvalue *)&cur_nod->nd_value);
 fprintf(lfile," (pdl_field %s",cur_nod->nd_name);
 	fprintf(lfile," %d %s",
 		(int)fld_vals.nd_size,
 		print_name[fld_vals.nd_print]);
 last_field_printas=fld_vals.nd_print;
 if(fld_vals.nd_indsize!=0)
 	fprintf(lfile," :size_from reg%d",(int)fld_vals.nd_size);
 if(fld_vals.nd_cntr!=0)
 	fprintf(lfile," :counter cnt%d",(int)fld_vals.nd_cntr);
 if(fld_vals.nd_indreg!=0)
 	fprintf(lfile," :value_from reg%d",(int)fld_vals.nd_indreg);
 if(fld_vals.nd_lodreg!=0)
 	fprintf(lfile," :value_to reg%d",(int)fld_vals.nd_lodreg);
 fprintf(lfile,")\n");
}

void lsp_print(nd_rec_pt);
void lsp_print(cur_nod)
nd_rec_pt cur_nod;
{switch(cur_nod->nd_type) {
 case IS_FLD:
  lsp_print_field(cur_nod);
  break;
 case IS_PROTO:
 case IS_NAME:
   fprintf(lfile,"  (pdl_name %s",
    cur_nod->nd_name);
   if(last_field_printas==PR_REAL(HEX))
    fprintf(lfile," 0x%lx",cur_nod->nd_value);
   else 
    fprintf(lfile," %ld",cur_nod->nd_value);

   if(cur_nod->nd_down!=0)
    fprintf(lfile," %s",cur_nod->nd_down->nd_name);
   else
    fprintf(lfile," nil");
   if(cur_nod->nd_genreg!=0)
   	fprintf(lfile," reg%d",(int)cur_nod->nd_genreg);
   else
    fprintf(lfile," nil");
   if((cur_nod->nd_cntr_flags&CNTR_scope_real)!=0)
    fprintf(lfile," :on_scope");
   if((cur_nod->nd_cntr_flags&CNTR_reject_real)!=0)
    fprintf(lfile," :reject");
   fprintf(lfile,")\n");
   break;
 case IS_INDGSUB:
  fprintf(lfile," (funcal reg%d)\n",
    	(int)cur_nod->nd_genreg);
  break;
 case IS_LOOPC:
  if(cur_nod->nd_down->nd_name==0)
   fprintf(lfile,
   " (loopc ???)\n");
  else
   fprintf(lfile,
   " (loopc reg%d '%s)\n",
   (int)cur_nod->nd_genreg,
   cur_nod->nd_down->nd_name);
  break;
 case IS_LOOPE:
  if(cur_nod->nd_down->nd_name==0)
   fprintf(lfile,
   " (loope ???)\n");
  else
   fprintf(lfile,
   " (loope '%s)\n",cur_nod->nd_down->nd_name);
  break;
 case IS_EOF:
  fprintf(lfile," (thats_all)\n");
  break;
 case IS_SPECIAL:
  fprintf(lfile," (funcal spcl%ld)\n",cur_nod->nd_value);
  break;
 case IS_END:
  fprintf(lfile,")\n\n");
  break;
 case IS_GOSUB:
  if(cur_nod->nd_down->nd_name==0)
   fprintf(lfile,
   " (funcal ???)\n");
  else
   fprintf(lfile,
   " (funcal '%s)\n",cur_nod->nd_down->nd_name);
  break;
 case IS_ZERO:
   fprintf(lfile,
   " (setq reg%d 0)\n",(int)cur_nod->nd_genreg);
  break;
 case IS_BEGIN:
  break;
 }
}

/* decode an opcode and print it to the .h file if it would be
   of any use there
*/
void print_res(nd_rec_pt,int*);
void print_res(cur_nod,bit_offset)
register nd_rec_pt cur_nod;
int *bit_offset;
{switch(cur_nod->nd_type) {
  /* name and proto opcodes define constants*/
  case IS_PROTO:
  case IS_NAME:
   fprintf(hfile,
   "#define %s (%ld)\n",cur_nod->nd_name,cur_nod->nd_value);
   break;
  /* field opcodes define structures components */
  case IS_FLD:
 	h_IS_FLD(cur_nod,bit_offset);
   break;
 }
}

/*see how many bits the current opcode allocates*/
void count_bits(nd_rec_pt,int *);
void count_bits(cur_nod,bit_offset)
register nd_rec_pt cur_nod;
int *bit_offset;
{nd_fvalue fld_vals;
 if((*bit_offset)<0)		/*bit offset unknown?*/
	return;					/*yes*/
 if(cur_nod->nd_type!=IS_FLD) /* is this a field opcode */
 	return;					/*no, nothing else allocates bits */
 fld_vals = *((nd_fvalue *)&cur_nod->nd_value); /* get our bits from nd_size */
 if(fld_vals.nd_indsize) {	/* size indirect? */
	*bit_offset = -1;		/*yes, the rest of the sizes are unknown */
	return;}
 *bit_offset +=fld_vals.nd_size; /* remember the allocated bits */
}

void do_cntr(nd_rec_pt);
void do_cntr(cur_nod)
register nd_rec_pt cur_nod;
{nd_fvalue fld_vals;
 register int the_counter;
 if(cur_nod->nd_type!=IS_FLD)	/*field opcode?*/
	return;			/*no, only they allocate counters*/
 fld_vals = *((nd_fvalue *)&cur_nod->nd_value);
 the_counter=fld_vals.nd_cntr;
 if(the_counter==0)	/*update a counter on this field?*/
	return;			/*no, don't */
 /*make sure the counter is in range*/
 assert((the_counter<MAX_COUNTS),
 	xat("counter %s is in range",cur_nod->nd_name));
 /*make shure this counter hasn't been used before*/
 assert((pdl_world->counts[the_counter]==0),
 	xat("counter %d not duplicate definiton at %s",
 		the_counter,cur_nod->nd_name));
 /*bind this counter to the current opcode
   this allows looking up a counter name and names for its
   values at run time
 */
 pdl_world->counts[the_counter]=cur_nod->meoff;
}
/*
	allocate the strings and space for each opcode
*/
void pdl_1pass(nd_rec_pt);
void pdl_1pass(cur_nod)
register nd_rec_pt cur_nod;
{int bit_offset;
 char *nod_name;
 nod_name=cur_nod->nd_name;
 /*if this record has a name, a c structure is desired for it*/
 if(nod_name!=NIL) {
 	fprintf(hfile,"struct %s_R {\n",nod_name);
 	fprintf(lfile,"(pdl_struct %s\n",nod_name);
 	bit_offset = 0;
 	}
 else
 	bit_offset = -1;
 do {
   cur_nod->meoff=cod_alloc;		/*my opcode will go here */
   cur_nod->choff=add_str(cur_nod); /*allocate space for this string */
   cod_alloc +=sizeof(nd_res);		/*allocte space for the opcode record*/
   assert(((cod_alloc&0x8000)==0),
   	xat("code segment is less than 32k"));
   do_cntr(cur_nod);				/*check for cur_nod being a counter*/
   lsp_print(cur_nod);				/* print to .lsp file */
   print_res(cur_nod,&bit_offset);	/*print to .h file*/
   count_bits(cur_nod,&bit_offset);	/*remember bits used*/
 } while ((cur_nod++)->nd_type!=IS_END);
 /*if we where making a structure, close it off*/
 if(nod_name!=NIL) {
 	fprintf(hfile,"};\n");
 	fprintf(hfile,"#define %s_size %d\n\n",nod_name,(bit_offset/8));
 	}
}

/*
	fill in each opcode in the current list, space is already allocated
	by pass1
*/
void pdl_2pass(nd_rec_pt);
void pdl_2pass(cur_nod)
register nd_rec_pt cur_nod;
{register int opcode;		/* opcode for current instruction */
 register nd_res *a_res;
 do {
   opcode=cur_nod->nd_type;		/* get the instruction to execute */
   if(op_recs[opcode].has_down)	{ /* this opcode have a down field? */
	if(cur_nod->nd_down->meoff==0)
      pdl_compl(cur_nod->nd_down); /* yes, so go parse it */
	cur_nod->dnoff=cur_nod->nd_down->meoff;
    }
   a_res = (nd_res *)(cod_ptr+cur_nod->meoff); /*where the opcode lives*/
   a_res->nd_type=cur_nod->nd_type;
   a_res->nd_genreg=cur_nod->nd_genreg;
   a_res->nd_value=cur_nod->nd_value;
   a_res->nd_cntr_flags=cur_nod->nd_cntr_flags;
   a_res->nd_down=cur_nod->dnoff;
   a_res->nd_str=cur_nod->choff;
 } while ((cur_nod++)->nd_type!=IS_END);
}

/* clear the memory pointed to astr */
void zero(char *,long);
void zero(astr,count)
register char *astr;
register long count;
{while(--count >=0)
	*astr++ = 0;
}

int remove(char *);

/*make a resource file and put the code and strings out to it*/
void put_reses(void);
void put_reses()
{int refnum;
 int oldref;
#define RES_FILE "pdl.rsrc"
#define PRES_FILE "\ppdl.rsrc"
 fprintf(stdout,"remove=%d\n",remove(RES_FILE));
 CreateResFile("\ppdl.rsrc");
 assert((ResError()==noErr),xat("can create %s",RES_FILE));
 oldref=CurResFile();
 refnum=OpenResFile(PRES_FILE);
 assert((ResError()==noErr),xat("can open %s",RES_FILE));
 fprintf(stdout,"OpenResFile=%d\n",refnum);
 UseResFile(refnum);
 AddResource(cod_handle,'PDLc',256,"\p");
 assert((ResError()==noErr),xat("can add to PDLc pdl resource file"));
 AddResource(str_handle,'PDLs',256,"\p");
 assert((ResError()==noErr),xat("can add to PDLs pdl resource file"));
 UseResFile(oldref);
 CloseResFile(refnum);
 assert((ResError()==noErr),xat("can close/update pdl resource file"));
}

/* make_space - allocate the space to compile into*/
void make_space(void);
void make_space()
{
/* segments can't be bigger then 32k, so HAN_SIZ is big enough*/
#define HAN_SIZ (40000L)
 cod_handle=(char **)NewHandle(HAN_SIZ);
 str_handle=(char **)NewHandle(HAN_SIZ);
 HLock(cod_handle);
 cod_ptr= *cod_handle;
 HLock(str_handle);
 str_ptr= *str_handle;
 zero(cod_ptr,HAN_SIZ);
 zero(str_ptr,HAN_SIZ);

 pdl_world = (pdl_world_hdr*)cod_ptr;	/*well known entry points table*/
 cod_alloc +=sizeof(pdl_world_hdr);
}

/* we are done allocating space, unlock the handles and
   set them to be the size of data actually used
*/
void finish_space(void);
void finish_space()
{
 HUnlock(cod_handle);
 HUnlock(str_handle);
 fprintf(stdout,"code size=%d\n",cod_alloc);
 fprintf(stdout,"str size=%d\n",str_alloc);
/*
	Cut the code and string resources down to their actual sizes
*/
 SetHandleSize(cod_handle,(uint32)cod_alloc);
 SetHandleSize(str_handle,(uint32)str_alloc);
}

void pdl_compl(cur_nod)
register nd_rec_pt cur_nod;
{if(cur_nod->meoff!=0)	/*recursing?*/
	return;				/*yes, don't*/
 pdl_1pass(cur_nod);	/*allocate space for the current record*/
 pdl_2pass(cur_nod);	/*recursively compile reachable nodes from cur_nod*/
}

void main(void);
void main()
{/* start out with big places to lay down the opcodes and strings
    we will cut them down to their actual sizes when we finish
 */
 fprintf(stdout,"pdl compiler v1.0 Apr/7/87\n");
 hfile=fopen("pdl_gen.h","w");
 fprintf(hfile,"/* this file is automatically generated */\n");
 lfile=fopen("pdl.lsp","w");
 make_space();	/* make the storage for compiling into*/
 pdl_compl(pdl_hdr);
 pdl_world->starts[GETPAK]=pdl_hdr[0].meoff;
 pdl_compl(pdl_root);	/*compile the pdl for appletalk*/
 pdl_world->starts[ATALK_root]=pdl_root[0].meoff;
 pdl_world->starts[IP_root]=ip[0].meoff;
 pdl_compl(misc_counts);	/*declare names for odd and ends counters*/
 finish_space();	/* finish with the resource, cut them to fit*/

 put_reses();		/*put code and string resources out the resource file*/
 assert((fclose(hfile)==0),xat("can close hfile"));
 assert((fclose(lfile)==0),xat("can close lfile"));
}
