/*
	seer packet filter/counter
*/
#include "sc.h"
#include "pdl_gen.h"
#include <asm.h>
#include "sc_filter.h"
#include "seer_interface.h"

/* filter procedure register usage
   d0 - free
   d1 - free
   d2.w - (preserv)errflag
   d3.w - (preserv)number of data bytes

   a0 - free
   a1 - (can trash)	address of seer driver globals (seer_data)
   a2 - (preserv)	address of end of packet + 1
   a3 - (can trash)	address start of packet
*/
/* define address registers*/
#define globs a1
#define pak a3			/*pointer to packet data */
/* data registers*/
#define errflag d2		/*error flags */
#define used d3			/*bytes used in packet */

#define KEEP_packet (0)
#define REJECT_packet (1)

#define CNT_OFFSET(tab_index) (OFFSET(filter_rec,counts)+tab_index*sizeof(cnt_tab))

/* called  from seer driver interrupt routine to count packets
   and see if they are to be accepted or rejected
*/
void packet_filter()
{asm{
	/* point to our data block, not the drivers globals*/
	move.l OFFSET(SR_status,filter_data)(globs),globs
	cmp.w #pdl_root_size,used	/*too short to be a lap packet?*/
	blt.s @tooshorter			/*yes, keep it then*/

/* process each of the three bytes of a lap packet.
   load each byte and use it for an index into a tab_tab table
   count the number of times the packet happens.  If the high order
   bit of a counter is set, then reject the packet.
*/
	move.l #0,d0			/*clear high bits*/
	move.b errflag,d0		/*get data byte*/
	asl.l #cnt_tab_shift,d0	/*make it an long word table index*/
	lea.l (CNT_OFFSET(err_tbi)+ET_PE_errs_offset)(globs),a0
	add.l #1,0(a0,d0.w)
	bmi.s @doreject

	move.l #0,d0			/*clear high bits*/
	move.b (pak)+,d0		/*get data byte*/
	asl.l #cnt_tab_shift,d0	/*make it an long word table index*/
	lea.l CNT_OFFSET(lap_dst_tbi)(globs),a0
	add.l #1,0(a0,d0.w)
	bmi.s @doreject

	move.l #0,d0			/*clear high bits*/
	move.b (pak)+,d0		/*get data byte*/
	asl.l #cnt_tab_shift,d0	/*make it an long word table index*/
	lea.l CNT_OFFSET(lap_src_tbi)(globs),a0
	add.l #1,0(a0,d0.w)
	bmi.s @doreject

	move.l #0,d0			/*clear high bits*/
	move.b (pak)+,d0		/*get data byte*/
	move.l d0,d1			/*remember for checking ddp type*/
	asl.l #cnt_tab_shift,d0	/*make it an long word table index*/
	lea.l CNT_OFFSET(lap_ty_tbi)(globs),a0
	add.l #1,0(a0,d0.w)
	bmi.s @doreject

/* assume the lap type says short ddp */
	move.l #(sddp_size-1),d0	/*offset from (pak) to ddp type*/
	cmpi.b #sddp,d1				/*is this a short ddp?*/
	beq.s @getddp				/*yes, go dispatch on ddp type*/

	move.l #(lddp_size-1),d0	/*assume long ddp*/
	cmpi.b #lddp,d1
	bne.s @dokeep

/* load the ddp type byte, if this is a ddp packet*/
getddp:
	move.b 0(pak,d0.w),d1	/*fetch the ddp type*/
	asl.l #cnt_tab_shift,d1	/*make it an long word table index*/
	add.w #3,d0				/*add lap overhead to ddp size*/
	cmp.w d0,used
	blt.s @tooshorter		/*yes, keep it then*/
	lea.l CNT_OFFSET(ddp_ty_tbi)(globs),a0
	add.l #1,0(a0,d1.w)
	bmi.s @doreject
	bra.s @dokeep

doreject:
	move.w #REJECT_packet,d0 	/*assume that we will reject the packet*/
	bra.s @doexit

tooshorter:
	lea.l CNT_OFFSET(err_tbi)(globs),a0
	add.l #1,ET_TOO_SHORT_offset(a0)
	bmi.s @doreject

dokeep:
	move.w #KEEP_packet,d0
doexit:
} /* end asm */
}
