/*SRr_main.c - 'seer' code resource entry point

Public domain promiscuous scc listener.  Public domain.  See
the read me file for more information.

Aaron Wohl
Carnegie -Mellon University
College of Fine Arts
Center for Art and Technology
Pittsburgh, PA 15213-3890
(412)-268-3454


Change log:

  date     who                   what
-------- -------- ---------------------------------------
02/22/87  wohl    create

 	defining eextern makes all seer static data be declared here and
   	not be external
*/
#define eextern
#include "seer_interface.h"
#include "seer_private.h"

/*this isn't really a routine, the address of the seer data is stored
   (patched into) here when seer starts up
*/
void real_sr_a4()
{asm {
	dc.l 0			/*0 means that the storage isn't setup yet */
	}
};

/*the table of control routines */
static int_rtn_ptr ctl_rtns[SRc_numberof];

/*map a csCode entry into a ctl_rtns array index */
#define OP_INDEX(xxx) ((xxx)+1)

/*since device drivers are relocatable and we want to use a
   dispatch table to the control routines for speed, we bind
   the relocatable addresses to absolute addresses here.
*/
void bind_rels(void);
void bind_rels()
{register int_rtn_ptr pt;
 ctl_rtns[OP_INDEX(-1)]=SRr_off;		 /*goodbye kiss */
 ctl_rtns[OP_INDEX(SRc_unused)]=SRr_off; /*unused/ reserved */
 ctl_rtns[OP_INDEX(SRc_killio)]=SRr_off; /*killio call */
 ctl_rtns[OP_INDEX(SRc_load)]=SRr_load;  /*init after loading driver */
 ctl_rtns[OP_INDEX(SRc_initq)]=SRr_initq; /*setup a data queue */
 ctl_rtns[OP_INDEX(SRc_off)]=SRr_off; 	 /*stop receiving data */
 ctl_rtns[OP_INDEX(SRc_on)]=SRr_on; 	 /*start receiving data */
 ctl_rtns[OP_INDEX(SRc_getpak)]=SRr_getpak; /*return a packet fm queue */
 ctl_rtns[OP_INDEX(SRc_release)]=SRr_release; /*delete a packet */
 ctl_rtns[OP_INDEX(SRc_clear_queue)]=SRr_clear_queue; /*empty the queue */
};


/*define the LightspeedC entry parameters for main */
#define LS_open		0
#define LS_prime	1
#define LS_control	2
#define LS_status	3
#define LS_close	4

/*
	LightSpeedC arranges for all device driver calls to come here.
	The 'entry' parameter specifies the type of call.
*/
int main(SR_iorecord *,DCtlPtr,int);
int main(pb, dce, entry)
register SR_iorecord *pb;
register DCtlPtr dce;
register int entry;
{asm {					/*setup a pointer to our global data */
	lea real_sr_a4,a0 	/*save data pointer for interrupt routines */
	move.l a4,(a0)
	};
 /*handle the different device driver entry points */
 switch(entry) {
 case LS_open:
	if (dce->dCtlStorage == 0) {
	  SysBeep(3);
	  return -23;		/*return openErr */
	  }
	/*we handle control, like goodbye, want to be locked */
	dce->dCtlFlags |= dNeedLock|dNeedGoodBye|dCtlEnable;
	/*we dont read, write or handle status calls */
	dce->dCtlFlags &= ~(dWritEnable|dReadEnable|dNeedTime);
	/*init sd varibles status=off, previous packets=0 */
	sd.ast.status=SR_status_off;
	sd.ast.pre_pak_num=0;
	sd.ast.h_inuse=0;
	sd.ast.h_size=0;
	sd.ast.d_inuse=0;
	bind_rels();		/*bind the addresses to relocatables */
	break;
 case LS_prime:
	return -19;			/*we shouldn't get read/writes */ 
 	break;
 case LS_control:
	{register int err,op;
	op=pb->io_CP.csCode;
	if((op< -1)||(op>=SRc_numberof)) /*is this a valid operation? */
		return -17;		/*we don't like this control call */
	err=(*ctl_rtns[OP_INDEX(op)])(pb->io_SR);
	if(op>SRc_killio)	/*have a real param block? */
		pb->io_SR->SR_error=err; /*yes, ok to set error code */
 	return noErr;
	break;
 	}
case LS_status:
	return -18;		/*we shouldn't get status errors */
 	break;
 case LS_close:
	dce->dCtlFlags &= ~(dNeedLock);	/*no longer need to be locked */
	if (dce->dCtlStorage == 0) /*if storage is not valid */
		return noErr;		/*nothing to clean up */
	(void)SRr_off(NIL);		/*remove interupt handlers */
 	break;
 default:
 	;
 }
 return noErr;
}